#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stm8l15x_adc.h>
#include <stm8l15x_tim1.h>
#include <stm8l15x_clk.h>
#include "globdef.h"
#include "mono_lcd.h"
#include "mallocR.h"

#define F_FILTER 50     //50Hz  
#define SAMPLES_PER_PERIOD 10 //F_FILTER periods to collect
#define PERIODS 50 //buffer for AD data storage
#define ADC_BUFFER_SIZE (SAMPLES_PER_PERIOD * PERIODS)

//collecting data from ADC into buffer
void ADCCollectDataFilt(unsigned int * ADCBuffer)
{
    unsigned int i;
    unsigned int divider;
    
    //timer init
    divider = F_CPU / SAMPLES_PER_PERIOD / F_FILTER;
    //enable clock for timer 1
    CLK_PeripheralClockConfig(CLK_Peripheral_TIM1, ENABLE);
    
    //set sampling frequency by timer
    TIM1_TimeBaseInit(0,
                       TIM1_CounterMode_Down,
                       divider,
                       0);
                       
    //TRGO enable on update event
    TIM1_SelectOutputTrigger(TIM1_TRGOSource_Update);
    TIM1_ARRPreloadConfig(ENABLE);
    //update timer
    TIM1_GenerateEvent(TIM1_EventSource_Update);
    
    //disable ADC interrupts
    ADC_ITConfig(ADC1, ADC_IT_EOC | ADC_IT_AWD | ADC_IT_OVER, DISABLE);
    //disable DMA for ADC
    ADC_DMACmd(ADC1, DISABLE);
    //init and enable ADC (one channel AIN_24)
    ADC_Init(ADC1,
              ADC_ConversionMode_Single,
              ADC_Resolution_12Bit,
              ADC_Prescaler_1);
    ADC_ChannelCmd(ADC1,
                    ADC_Channel_24,
                    ENABLE);
    ADC_SchmittTriggerConfig(ADC1,
                    ADC_Channel_24,
                    ENABLE);
    ADC_SamplingTimeConfig(ADC1,
                            ADC_Group_FastChannels, //channel 24  is in fast group
                            ADC_SamplingTime_16Cycles);
    ADC_ExternalTrigConfig(ADC1,
                            ADC_ExtEventSelection_Trigger2, //for timer1 event is selected trigger2
                            ADC_ExtTRGSensitivity_Rising);
                              
    //start timer - periodic trigger 
    TIM1_Cmd(ENABLE);
    
    //sample data
    i=0;
    while (i<ADC_BUFFER_SIZE)
    {
      //wait for EOC
      while (!ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC));
      //collect ADC value
      ADCBuffer [i] = ADC_GetConversionValue(ADC1);
      i++;
    }

    //stop data sampling
    TIM1_Cmd(DISABLE);  //stop timer - trigger
}//ADCCollectData
//performs digital filter on collected ADC data
//comb filter on 50Hz
static void Filter50Hz(unsigned int * ADCBuffer)
{
  //DIGITAL FILTER IMPLEMENTATION (comb filter for given frequency): 
  //       [1-z(-N)]
  // H(z)= ---------                    N... samples per period
  //       [1-z(-1)]
  unsigned int i,j;
  unsigned long * ACC;
  unsigned long IntegratorOut;
  

  //allocate accumulator space
  if ((ACC = malloc(SAMPLES_PER_PERIOD * sizeof(ACC[0]))) == NULL)
    trap();
  
  //init ACC to first sample
  for (i=0; i< SAMPLES_PER_PERIOD; i++)
    ACC[i]=ADCBuffer[0];
  IntegratorOut=ADCBuffer[0];
  
  //filtering
  for (i=0; i<ADC_BUFFER_SIZE; i++)
  {
    for(j=SAMPLES_PER_PERIOD-1; j>0 ; j--)
      ACC[j]=ACC[j-1];
    ACC[0]=IntegratorOut;
    IntegratorOut += ADCBuffer[i];    
    
    //write back the filtered signal
    ADCBuffer[i]= (IntegratorOut - ACC[SAMPLES_PER_PERIOD-1])/SAMPLES_PER_PERIOD;
  }
  free(ACC) ;
}//Filter50Hz
//returns min and max value difference from buffer
//for 50Hz suppression testing
unsigned int GetMinMaxFilt(unsigned int * Buffer, unsigned int buffersize)
{
  unsigned int i,min,max;
  
  min=0xFFFF;
  max=0;
  for (i=0; i<buffersize; i++)
  {
    if (Buffer[i]<min)
      min = Buffer[i];
    if (Buffer[i]>max)
      max = Buffer[i];
  }
  return(max-min);
}
//main routine for digital filter testing
void TestADCDigitalFilter50Hz(void)
{
  unsigned int * ADCBuffer;
  volatile unsigned int diffOrig;
  volatile unsigned int diffFilt;
  
  //allocate buffer space
  if ((ADCBuffer = malloc(ADC_BUFFER_SIZE * sizeof(ADCBuffer[0]))) == NULL)
    trap();
  
  //collect data
  ADCCollectDataFilt(ADCBuffer);
  diffOrig = GetMinMaxFilt(&ADCBuffer[SAMPLES_PER_PERIOD], ADC_BUFFER_SIZE - SAMPLES_PER_PERIOD);
 
  //filter data
  Filter50Hz(ADCBuffer);
  diffFilt = GetMinMaxFilt(&ADCBuffer[SAMPLES_PER_PERIOD], ADC_BUFFER_SIZE - SAMPLES_PER_PERIOD);    
  
  LCD_PrintString(LCD_LINE1, DISABLE, DISABLE, "dOrig50Hz=");
  LCD_PrintDec2(diffOrig/100);
  LCD_PrintDec2(diffOrig%100);
  
  LCD_PrintString(LCD_LINE2, DISABLE, DISABLE, "dFilt50Hz=");
  LCD_PrintDec2(diffFilt/100);
  LCD_PrintDec2(diffFilt%100);
  
  //unallocate buffer
  free(ADCBuffer);
}//TestADCDigitalFilter50Hz

